'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var classPrivateMethodGet = require('../../../dist/classPrivateMethodGet-2e59296b.cjs.dev.js');
var getProvider = require('../../../dist/getProvider-e9de0cc3.cjs.dev.js');
var utils = require('ethers/lib/utils');
var ethers = require('ethers');
var EventEmitter = require('eventemitter3');
require('zustand/middleware');
require('zustand/vanilla');
require('../../../dist/chains-bafea3b1.cjs.dev.js');
require('../../../dist/rpcs-0d21dfe8.cjs.dev.js');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

var EventEmitter__default = /*#__PURE__*/_interopDefault(EventEmitter);

class MockProvider extends ethers.providers.BaseProvider {
  constructor(options) {
    super({
      name: 'Network',
      chainId: options.chainId ?? 1
    });
    getProvider._defineProperty(this, "events", new EventEmitter__default["default"]());
    getProvider._defineProperty(this, "options", void 0);
    getProvider._defineProperty(this, "signer", void 0);
    this.options = options;
  }
  async enable() {
    var _this$options$flags;
    if ((_this$options$flags = this.options.flags) !== null && _this$options$flags !== void 0 && _this$options$flags.failConnect) throw new getProvider.UserRejectedRequestError(new Error('Failed to connect'));
    if (!this.signer) this.signer = this.options.signer;
    const address = await this.signer.getAddress();
    this.events.emit('accountsChanged', [address]);
    return [address];
  }
  async disconnect() {
    this.events.emit('disconnect');
    this.signer = undefined;
  }
  async getAccounts() {
    var _this$signer;
    const address = await ((_this$signer = this.signer) === null || _this$signer === void 0 ? void 0 : _this$signer.getAddress());
    if (!address) return [];
    return [utils.getAddress(address)];
  }
  getSigner() {
    const signer = this.signer;
    if (!signer) throw new Error('Signer not found');
    return signer;
  }
  async switchChain(chainId) {
    var _this$options$flags2;
    if ((_this$options$flags2 = this.options.flags) !== null && _this$options$flags2 !== void 0 && _this$options$flags2.failSwitchChain) throw new getProvider.UserRejectedRequestError(new Error('Failed to switch chain'));
    this.options.chainId = chainId;
    this.network.chainId = chainId;
    this.events.emit('chainChanged', chainId);
  }
  async watchAsset(_asset) {
    return true;
  }
  on(event, listener) {
    this.events.on(event, listener);
    return this;
  }
  once(event, listener) {
    this.events.once(event, listener);
    return this;
  }
  removeListener(event, listener) {
    this.events.removeListener(event, listener);
    return this;
  }
  off(event, listener) {
    this.events.off(event, listener);
    return this;
  }
  toJSON() {
    return '<MockProvider>';
  }
}

var _provider = /*#__PURE__*/new WeakMap();
var _switchChain = /*#__PURE__*/new WeakSet();
class MockConnector extends getProvider.Connector {
  constructor(config) {
    super(config);
    classPrivateMethodGet._classPrivateMethodInitSpec(this, _switchChain);
    getProvider._defineProperty(this, "id", 'mock');
    getProvider._defineProperty(this, "name", 'Mock');
    getProvider._defineProperty(this, "ready", true);
    classPrivateMethodGet._classPrivateFieldInitSpec(this, _provider, {
      writable: true,
      value: void 0
    });
    getProvider._defineProperty(this, "onAccountsChanged", accounts => {
      if (accounts.length === 0) this.emit('disconnect');else this.emit('change', {
        account: utils.getAddress(accounts[0])
      });
    });
    getProvider._defineProperty(this, "onChainChanged", chainId => {
      const id = getProvider.normalizeChainId(chainId);
      const unsupported = this.isChainUnsupported(id);
      this.emit('change', {
        chain: {
          id,
          unsupported
        }
      });
    });
    getProvider._defineProperty(this, "onDisconnect", () => {
      this.emit('disconnect');
    });
  }
  async connect() {
    var _this$options$flags;
    let {
      chainId
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    const provider = await this.getProvider({
      chainId
    });
    provider.on('accountsChanged', this.onAccountsChanged);
    provider.on('chainChanged', this.onChainChanged);
    provider.on('disconnect', this.onDisconnect);
    this.emit('message', {
      type: 'connecting'
    });
    const accounts = await provider.enable();
    const account = utils.getAddress(accounts[0]);
    const id = getProvider.normalizeChainId(provider._network.chainId);
    const unsupported = this.isChainUnsupported(id);
    const data = {
      account,
      chain: {
        id,
        unsupported
      },
      provider
    };
    if (!((_this$options$flags = this.options.flags) !== null && _this$options$flags !== void 0 && _this$options$flags.noSwitchChain)) this.switchChain = classPrivateMethodGet._classPrivateMethodGet(this, _switchChain, _switchChain2);
    return new Promise(res => setTimeout(() => res(data), 100));
  }
  async disconnect() {
    const provider = await this.getProvider();
    await provider.disconnect();
    provider.removeListener('accountsChanged', this.onAccountsChanged);
    provider.removeListener('chainChanged', this.onChainChanged);
    provider.removeListener('disconnect', this.onDisconnect);
  }
  async getAccount() {
    const provider = await this.getProvider();
    const accounts = await provider.getAccounts();
    const account = accounts[0];
    if (!account) throw new Error('Failed to get account');
    // return checksum address
    return utils.getAddress(account);
  }
  async getChainId() {
    const provider = await this.getProvider();
    return getProvider.normalizeChainId(provider.network.chainId);
  }
  async getProvider() {
    let {
      chainId
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    if (!classPrivateMethodGet._classPrivateFieldGet(this, _provider) || chainId) classPrivateMethodGet._classPrivateFieldSet(this, _provider, new MockProvider({
      ...this.options,
      chainId
    }));
    return classPrivateMethodGet._classPrivateFieldGet(this, _provider);
  }
  async getSigner() {
    const provider = await this.getProvider();
    return provider.getSigner();
  }
  async isAuthorized() {
    try {
      var _this$options$flags2;
      const provider = await this.getProvider();
      const account = await provider.getAccounts();
      return ((_this$options$flags2 = this.options.flags) === null || _this$options$flags2 === void 0 ? void 0 : _this$options$flags2.isAuthorized) ?? !!account;
    } catch {
      return false;
    }
  }
  async watchAsset(asset) {
    const provider = await this.getProvider();
    return provider.watchAsset(asset);
  }
  toJSON() {
    return '<MockConnector>';
  }
}
async function _switchChain2(chainId) {
  const provider = await this.getProvider();
  await provider.switchChain(chainId);
  return this.chains.find(x => x.id === chainId) ?? {
    id: chainId,
    name: `Chain ${chainId}`,
    network: `${chainId}`,
    rpcUrls: {
      default: ''
    }
  };
}

exports.MockConnector = MockConnector;
exports.MockProvider = MockProvider;
