import { _ as _classPrivateMethodInitSpec, a as _classPrivateFieldInitSpec, b as _classPrivateMethodGet, c as _classPrivateFieldGet, d as _classPrivateFieldSet } from '../../../dist/classPrivateMethodGet-9a97772b.esm.js';
import { _ as _defineProperty, U as UserRejectedRequestError, l as Connector, n as normalizeChainId } from '../../../dist/getProvider-54742edd.esm.js';
import { getAddress } from 'ethers/lib/utils';
import { providers } from 'ethers';
import EventEmitter from 'eventemitter3';
import 'zustand/middleware';
import 'zustand/vanilla';
import '../../../dist/chains-e9e4eb3b.esm.js';
import '../../../dist/rpcs-b4aa70b9.esm.js';

class MockProvider extends providers.BaseProvider {
  constructor(options) {
    super({
      name: 'Network',
      chainId: options.chainId ?? 1
    });
    _defineProperty(this, "events", new EventEmitter());
    _defineProperty(this, "options", void 0);
    _defineProperty(this, "signer", void 0);
    this.options = options;
  }
  async enable() {
    var _this$options$flags;
    if ((_this$options$flags = this.options.flags) !== null && _this$options$flags !== void 0 && _this$options$flags.failConnect) throw new UserRejectedRequestError(new Error('Failed to connect'));
    if (!this.signer) this.signer = this.options.signer;
    const address = await this.signer.getAddress();
    this.events.emit('accountsChanged', [address]);
    return [address];
  }
  async disconnect() {
    this.events.emit('disconnect');
    this.signer = undefined;
  }
  async getAccounts() {
    var _this$signer;
    const address = await ((_this$signer = this.signer) === null || _this$signer === void 0 ? void 0 : _this$signer.getAddress());
    if (!address) return [];
    return [getAddress(address)];
  }
  getSigner() {
    const signer = this.signer;
    if (!signer) throw new Error('Signer not found');
    return signer;
  }
  async switchChain(chainId) {
    var _this$options$flags2;
    if ((_this$options$flags2 = this.options.flags) !== null && _this$options$flags2 !== void 0 && _this$options$flags2.failSwitchChain) throw new UserRejectedRequestError(new Error('Failed to switch chain'));
    this.options.chainId = chainId;
    this.network.chainId = chainId;
    this.events.emit('chainChanged', chainId);
  }
  async watchAsset(_asset) {
    return true;
  }
  on(event, listener) {
    this.events.on(event, listener);
    return this;
  }
  once(event, listener) {
    this.events.once(event, listener);
    return this;
  }
  removeListener(event, listener) {
    this.events.removeListener(event, listener);
    return this;
  }
  off(event, listener) {
    this.events.off(event, listener);
    return this;
  }
  toJSON() {
    return '<MockProvider>';
  }
}

var _provider = /*#__PURE__*/new WeakMap();
var _switchChain = /*#__PURE__*/new WeakSet();
class MockConnector extends Connector {
  constructor(config) {
    super(config);
    _classPrivateMethodInitSpec(this, _switchChain);
    _defineProperty(this, "id", 'mock');
    _defineProperty(this, "name", 'Mock');
    _defineProperty(this, "ready", true);
    _classPrivateFieldInitSpec(this, _provider, {
      writable: true,
      value: void 0
    });
    _defineProperty(this, "onAccountsChanged", accounts => {
      if (accounts.length === 0) this.emit('disconnect');else this.emit('change', {
        account: getAddress(accounts[0])
      });
    });
    _defineProperty(this, "onChainChanged", chainId => {
      const id = normalizeChainId(chainId);
      const unsupported = this.isChainUnsupported(id);
      this.emit('change', {
        chain: {
          id,
          unsupported
        }
      });
    });
    _defineProperty(this, "onDisconnect", () => {
      this.emit('disconnect');
    });
  }
  async connect() {
    var _this$options$flags;
    let {
      chainId
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    const provider = await this.getProvider({
      chainId
    });
    provider.on('accountsChanged', this.onAccountsChanged);
    provider.on('chainChanged', this.onChainChanged);
    provider.on('disconnect', this.onDisconnect);
    this.emit('message', {
      type: 'connecting'
    });
    const accounts = await provider.enable();
    const account = getAddress(accounts[0]);
    const id = normalizeChainId(provider._network.chainId);
    const unsupported = this.isChainUnsupported(id);
    const data = {
      account,
      chain: {
        id,
        unsupported
      },
      provider
    };
    if (!((_this$options$flags = this.options.flags) !== null && _this$options$flags !== void 0 && _this$options$flags.noSwitchChain)) this.switchChain = _classPrivateMethodGet(this, _switchChain, _switchChain2);
    return new Promise(res => setTimeout(() => res(data), 100));
  }
  async disconnect() {
    const provider = await this.getProvider();
    await provider.disconnect();
    provider.removeListener('accountsChanged', this.onAccountsChanged);
    provider.removeListener('chainChanged', this.onChainChanged);
    provider.removeListener('disconnect', this.onDisconnect);
  }
  async getAccount() {
    const provider = await this.getProvider();
    const accounts = await provider.getAccounts();
    const account = accounts[0];
    if (!account) throw new Error('Failed to get account');
    // return checksum address
    return getAddress(account);
  }
  async getChainId() {
    const provider = await this.getProvider();
    return normalizeChainId(provider.network.chainId);
  }
  async getProvider() {
    let {
      chainId
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    if (!_classPrivateFieldGet(this, _provider) || chainId) _classPrivateFieldSet(this, _provider, new MockProvider({
      ...this.options,
      chainId
    }));
    return _classPrivateFieldGet(this, _provider);
  }
  async getSigner() {
    const provider = await this.getProvider();
    return provider.getSigner();
  }
  async isAuthorized() {
    try {
      var _this$options$flags2;
      const provider = await this.getProvider();
      const account = await provider.getAccounts();
      return ((_this$options$flags2 = this.options.flags) === null || _this$options$flags2 === void 0 ? void 0 : _this$options$flags2.isAuthorized) ?? !!account;
    } catch {
      return false;
    }
  }
  async watchAsset(asset) {
    const provider = await this.getProvider();
    return provider.watchAsset(asset);
  }
  toJSON() {
    return '<MockConnector>';
  }
}
async function _switchChain2(chainId) {
  const provider = await this.getProvider();
  await provider.switchChain(chainId);
  return this.chains.find(x => x.id === chainId) ?? {
    id: chainId,
    name: `Chain ${chainId}`,
    network: `${chainId}`,
    rpcUrls: {
      default: ''
    }
  };
}

export { MockConnector, MockProvider };
