import { Address } from 'abitype';
import { providers } from 'ethers';
import { Signer } from '../../types';
export declare type PrepareSendTransactionArgs<TSigner extends Signer = Signer> = {
    /** Chain ID used to validate if the signer is connected to the target chain */
    chainId?: number;
    /** Request data to prepare the transaction */
    request: providers.TransactionRequest & {
        to: NonNullable<providers.TransactionRequest['to']>;
    };
    signer?: TSigner | null;
};
export declare type PrepareSendTransactionResult = {
    chainId?: number;
    request: providers.TransactionRequest & {
        to: Address;
        gasLimit: NonNullable<providers.TransactionRequest['gasLimit']>;
    };
    mode: 'prepared';
};
/**
 * @description Prepares the parameters required for sending a transaction.
 *
 * Returns config to be passed through to `sendTransaction`.
 *
 * @example
 * import { prepareSendTransaction, sendTransaction } from '@vue-ethereum-hooks/core'
 *
 * const config = await prepareSendTransaction({
 *  request: {
 *    to: 'moxey.eth',
 *    value: parseEther('1'),
 *  }
 * })
 * const result = await sendTransaction(config)
 */
export declare function prepareSendTransaction({ chainId, request, signer: signer_, }: PrepareSendTransactionArgs): Promise<PrepareSendTransactionResult>;
